@@ -1,4 +1,4 @@
-/* $Id: default.c,v 3.96 1995/02/13 10:56:37 bert Exp $
+/* $Id: default.c,v 3.110 1996/05/04 21:43:44 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -30,12 +30,10 @@
 #    include <X11/ap_keysym.h>
 #endif
 #include <sys/types.h>
+#include <unistd.h>
 #ifdef VMS
 #include "strcasecmp.h"
-#include <unixio.h>
-#include <unixlib.h>
 #else
-#include <unistd.h>
 #include <sys/param.h>
 #endif
 #include <string.h>
@@ -59,7 +57,7 @@ char default_version[] = VERSION;
 
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: default.c,v 3.96 1995/02/13 10:56:37 bert Exp $";
+    "@(#)$Id: default.c,v 3.110 1996/05/04 21:43:44 bert Exp $";
 #endif
 
 #ifdef VMS
@@ -92,6 +90,11 @@ static char		*myName = "xpilot";
 static char		*myClass = "XPilot";
 
 
+#ifdef SPARC_CMAP_HACK
+char  frameBuffer[MAX_CHARS]; /* frame buffer */
+#endif
+
+
 keys_t buttonDefs[MAX_POINTER_BUTTONS];
 
 /*
@@ -223,7 +226,7 @@ struct option {
     {
 	"shipShapeFile",
 	NULL,
-	"",
+	SHIP_FILE,
 	KEY_DUMMY,
 	"An optional file where shipshapes can be stored.\n"
 	"If this resource is defined and it refers to an existing file\n"
@@ -484,6 +487,13 @@ struct option {
 	KEY_DUMMY,
 	"Should a clock be displayed in the top right of the score window.\n"
     },
+    {
+	"clockAMPM",
+	NULL,
+	"No",
+	KEY_DUMMY,
+	"12 or 24 hour format for clock display.\n",
+    },
     {
 	"pointerControl",
 	NULL,
@@ -592,6 +602,13 @@ struct option {
 	KEY_DUMMY,
 	"Are shields toggled by a keypress only?\n"
     },
+    {
+	"autoShield", /* Don auto-shield hack */
+	NULL,
+	"Yes",
+	KEY_DUMMY,
+	"Are shields lowered automatically for weapon fire?\n"
+    },
     {
 	"shieldDrawSolid",
 	NULL,
@@ -668,6 +685,13 @@ struct option {
 	KEY_DUMMY,
 	"Use color buffering or not.\n"
     },
+    {
+	"multibuffer",
+	NULL,
+	"No",
+	KEY_DUMMY,
+	"Use the X windows multibuffer extension if present.\n"
+    },
     {
 	"maxColors",
 	NULL,
@@ -848,13 +872,86 @@ struct option {
 	KEY_DUMMY,
 	"Which color number to use for drawing walls.\n"
     },
+    {
+	"wallRadarColor",
+	NULL,
+	"2",
+	KEY_DUMMY,
+	"Which color number to use for drawing walls on the radar.\n"
+	"Valid values all even numbers smaller than maxColors.\n"
+    },
+    {
+	"decorColor",
+	NULL,
+	"6",
+	KEY_DUMMY,
+	"Which color number to use for drawing decorations.\n"
+    },
+    {
+	"showDecor",
+	NULL,
+	"Yes",
+	KEY_DUMMY,
+	"Should decorations be displayed on the screen and radar?\n"
+    },
+    {
+	"decorRadarColor",
+	NULL,
+	"6",
+	KEY_DUMMY,
+	"Which color number to use for drawing decorations on the radar.\n"
+	"Valid values are all even numbers smaller than maxColors.\n"
+    },
+    {
+	"outlineDecor",
+	NULL,
+	"No",
+	KEY_DUMMY,
+	"Draws only the outline of the map decoration.\n"
+    },
+    {
+	"filledDecor",
+	NULL,
+	"No",
+	KEY_DUMMY,
+	"Draws filled decorations.\n"
+    },
+    {
+	"texturedDecor",
+	NULL,
+	"No",
+	KEY_DUMMY,
+	"Draws the map decoration filled with a texture pattern.\n"
+	"See also the decorTextureFile and texturedWalls options.\n"
+    },
+    {
+	"decorTextureFile",
+	NULL,
+	"",
+	KEY_DUMMY,
+	"Specify a XPM format pixmap file to load the decor texture from.\n"
+    },
+    {
+	"texturedBalls",
+	NULL,
+	"No",
+	KEY_DUMMY,
+	"Draw the balls with a texture specified by the ballTextureFile option.\n"
+    },
+    {
+	"ballTextureFile",
+	NULL,
+	"",
+	KEY_DUMMY,
+	"Specify a XPM format pixmap file to load the ball texture from.\n"
+    },
     {
 	"targetRadarColor",
 	NULL,
 	"4",
 	KEY_DUMMY,
 	"Which color number to use for drawing targets on the radar.\n"
-	"Valid values all powers of 2 smaller than maxColors.\n"
+	"Valid values are all even numbers smaller than maxColors.\n"
     },
     {
 	"sparkColors",
@@ -1336,6 +1433,20 @@ struct option {
 	KEY_TOGGLE_RECORD,
 	"Toggle recording of session (see recordFile).\n"
     },
+    {
+	"keySelectItem",
+	NULL,
+	"KP_0 KP_Insert",
+	KEY_SELECT_ITEM,
+	"Select an item to lose.\n"
+    },
+    {
+	"keyLoseItem",
+	NULL,
+	"KP_Delete KP_Decimal",
+	KEY_LOSE_ITEM,
+	"Lose the selected item.\n"
+    },
     {
 	"keyPointerControl",
 	NULL,
@@ -1416,6 +1527,16 @@ struct option {
 	"Specifies the audio server to use.\n"
     },
 #endif
+#ifdef SPARC_CMAP_HACK
+    {
+	"frameBuffer",
+	NULL,
+	"/dev/fb",
+	KEY_DUMMY,
+	"Specifies the device name of the frame buffer.\n"
+    },
+#endif    
+
 };
 
 
@@ -1499,7 +1620,7 @@ static void Usage(void)
 "Each key option may have mutliple keys bound to it and\n"
 "one key may be used by multiple key options.\n"
 "If no server is specified then xpilot will search\n"
-"for servers on your local network\n"
+"for servers on your local network.\n"
 "For a listing of remote servers try: telnet xpilot.cs.uit.no 4400 \n"
 	  );
 
@@ -1740,7 +1861,11 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     KeySym		ks;
     char		*ptr,
 			*str,
+#ifdef VMS
+			resValue[PATH_MAX + 1];
+#else
 			resValue[MAX(2*MSG_LEN, PATH_MAX + 1)];
+#endif
     XrmDatabase		argDB = 0, rDB;
     XrmOptionDescRec	*xopt;
     extern void		Record_init(char *filename);
@@ -1963,6 +2088,7 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     else
 	shieldDrawMode = ON(resValue);
     Get_bool_resource(rDB, "toggleShield", &toggle_shield);
+    Get_bool_resource(rDB, "autoShield", &auto_shield);
 
     Get_resource(rDB, "modifierBank1", modBankStr[0], sizeof modBankStr[0]);
     Get_resource(rDB, "modifierBank2", modBankStr[1], sizeof modBankStr[1]);
@@ -1974,6 +2100,8 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     mono = (i != 0) ? true : false;
     Get_bool_resource(rDB, "colorSwitch", &i);
     colorSwitch = (i != 0) ? true : false;
+    Get_bool_resource(rDB, "multibuffer", &i);
+    multibuffer = (i != 0) ? true : false;
     Get_int_resource(rDB, "maxColors", &maxColors);
     Get_string_resource(rDB, "black", color_names[0], sizeof(color_names[0]));
     Get_string_resource(rDB, "white", color_names[1], sizeof(color_names[1]));
@@ -1992,6 +2120,9 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     Get_int_resource(rDB, "hudColor", &hudColor);
     Get_int_resource(rDB, "hudLockColor", &hudLockColor);
     Get_int_resource(rDB, "wallColor", &wallColor);
+    Get_int_resource(rDB, "wallRadarColor", &wallRadarColor);
+    Get_int_resource(rDB, "decorColor", &decorColor);
+    Get_int_resource(rDB, "decorRadarColor", &decorRadarColor);
     Get_int_resource(rDB, "targetRadarColor", &targetRadarColor);
     Get_resource(rDB, "sparkColors", sparkColors, MSG_LEN);
 
@@ -2012,9 +2143,15 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     Get_bit_resource(rDB, "slidingRadar", &instruments, SHOW_SLIDING_RADAR);
     Get_bit_resource(rDB, "showItems", &instruments, SHOW_ITEMS);
     Get_bit_resource(rDB, "clock", &instruments, SHOW_CLOCK);
+    Get_bit_resource(rDB, "clockAMPM", &instruments, SHOW_CLOCK_AMPM_FORMAT);
     Get_bit_resource(rDB, "outlineWorld", &instruments, SHOW_OUTLINE_WORLD);
     Get_bit_resource(rDB, "filledWorld", &instruments, SHOW_FILLED_WORLD);
     Get_bit_resource(rDB, "texturedWalls", &instruments, SHOW_TEXTURED_WALLS);
+    Get_bit_resource(rDB, "showDecor", &instruments, SHOW_DECOR);
+    Get_bit_resource(rDB, "outlineDecor", &instruments, SHOW_OUTLINE_DECOR);
+    Get_bit_resource(rDB, "filledDecor", &instruments, SHOW_FILLED_DECOR);
+    Get_bit_resource(rDB, "texturedDecor", &instruments, SHOW_TEXTURED_DECOR);
+    Get_bit_resource(rDB, "texturedBalls", &instruments, SHOW_TEXTURED_BALLS);
 
     Get_bool_resource(rDB, "pointerControl", &initialPointerControl);
     Get_float_resource(rDB, "showItemsTime", &showItemsTime);
@@ -2043,6 +2180,10 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     texturePath = strdup(resValue);
     Get_resource(rDB, "wallTextureFile", resValue, sizeof resValue);
     wallTextureFile = strdup(resValue);
+    Get_resource(rDB, "decorTextureFile", resValue, sizeof resValue);
+    decorTextureFile = strdup(resValue);
+    Get_resource(rDB, "ballTextureFile", resValue, sizeof resValue);
+    ballTextureFile = strdup(resValue);
 
     Get_int_resource(rDB, "maxFPS", &maxFPS);
     oldMaxFPS = maxFPS;
@@ -2053,6 +2194,10 @@ void Parse_options(int *argcp, char **argvp, char *realName, int *port,
     Get_resource(rDB, "audioServer", audioServer, sizeof audioServer);
 #endif
 
+#ifdef SPARC_CMAP_HACK
+    Get_string_resource(rDB, "frameBuffer", frameBuffer, sizeof frameBuffer);
+#endif    
+
     /*
      * Key bindings
      */
