@@ -1,4 +1,4 @@
-/* $Id: join.c,v 3.27 1995/01/11 19:32:34 bert Exp $
+/* $Id: join.c,v 3.29 1996/05/02 16:05:47 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -22,25 +22,15 @@
  */
 
 #include "types.h"
-#ifdef VMS
-#include <unixio.h>
-#include <unixlib.h>
-#else
 #include <unistd.h>
-#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <signal.h>
 #include <time.h>
-#ifdef VMS
-#include <socket.h>
-#include <in.h>
-#else
 #include <sys/socket.h>
 #include <netinet/in.h>
-#endif
 #include <netdb.h>
-#if !defined(__hpux) && !defined(VMS)
+#if !defined(__hpux) 
 #include <sys/time.h>
 #endif
 
@@ -90,6 +80,7 @@ void Input_loop(void)
 	error("Bad socket filedescriptor");
 	return;
     }
+    Net_key_change();
     rfds = 0;
     rfds |= (1 << clientfd);
     rfds |= (1 << netfd);
