@@ -1,4 +1,4 @@
-/* $Id: net.c,v 3.29 1995/01/11 19:36:09 bert Exp $
+/* $Id: net.c,v 3.31 1996/05/02 16:05:49 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -21,28 +21,18 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifdef VMS
-#include <unixio.h>
-#include <unixlib.h>
-#else
 #include <unistd.h>
-#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
 #include <sys/types.h>
-#if defined(__hpux) || defined(VMS)
+#if defined(__hpux)
 #include <time.h>
 #else
 #include <sys/time.h>
 #endif
-#ifdef VMS
-#include <socket.h>
-#include <in.h>
-#else
 #include <sys/socket.h>
 #include <netinet/in.h>
-#endif
 #include <netdb.h>
 #ifdef __sgi
 #include <bstring.h>
@@ -241,7 +231,7 @@ int Sockbuf_flush(sockbuf_t *sbuf)
 	Sockbuf_clear(sbuf);
     } else {
 	errno = 0;
-	while ((len = write(sbuf->sock, sbuf->buf, sbuf->len)) <= 0) {
+	while ((len = DgramWrite(sbuf->sock, sbuf->buf, sbuf->len)) <= 0) {
 	    if (errno == EINTR) {
 		errno = 0;
 		continue;
@@ -319,7 +309,7 @@ int Sockbuf_read(sockbuf_t *sbuf)
 	    len = sbuf->len;
 	else
 #endif
-	while ((len = recv(sbuf->sock, sbuf->buf + sbuf->len, max, 0)) <= 0) {
+	while ((len = DgramRead(sbuf->sock, sbuf->buf + sbuf->len, max)) <= 0) {
 	    if (len == 0) {
 		return 0;
 	    }
@@ -355,7 +345,7 @@ int Sockbuf_read(sockbuf_t *sbuf)
 	sbuf->len += len;
     } else {
 	errno = 0;
-	while ((len = read(sbuf->sock, sbuf->buf + sbuf->len, max)) <= 0) {
+	while ((len = DgramRead(sbuf->sock, sbuf->buf + sbuf->len, max)) <= 0) {
 	    if (len == 0) {
 		return 0;
 	    }
