@@ -1,4 +1,4 @@
-/* $Id: netclient.c,v 3.82 1995/01/29 16:17:44 bert Exp $
+/* $Id: netclient.c,v 3.92 1996/05/13 12:16:48 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -22,14 +22,11 @@
  */
 
 #include "types.h"
-#ifdef VMS
-#include <unixio.h>
-#include <unixlib.h>
-#else
 #include <unistd.h>
+#ifndef VMS
 #include <sys/param.h>
 #endif
-#if defined(__hpux) || defined(VMS)
+#if defined(__hpux)
 #include <time.h>
 #else
 #include <sys/time.h>
@@ -150,6 +147,7 @@ static void Receive_init(void)
     receive_tbl[PKT_FASTSHOT]	= Receive_fastshot;
     receive_tbl[PKT_THRUSTTIME] = Receive_thrusttime;
     receive_tbl[PKT_SHIELDTIME] = Receive_shieldtime;
+    receive_tbl[PKT_LOSEITEM]	= Receive_loseitem;
     for (i = 0; i < DEBRIS_TYPES; i++) {
 	receive_tbl[PKT_DEBRIS + i] = Receive_debris;
     }
@@ -510,7 +508,7 @@ int Net_init(char *server, int port)
     }
     if (DgramConnect(sock, server, port) == -1) {
 	error("Can't connect to server %s on port %d", server, port);
-	close(sock);
+	DgramClose(sock);
 	return -1;
     }
     wbuf.sock = sock;
@@ -576,13 +574,13 @@ void Net_cleanup(void)
 {
     int		i,
 		sock = wbuf.sock;
-    u_byte	ch;
+    char	ch;
 
     if (sock > 2) {
 	ch = PKT_QUIT;
-	if (write(sock, &ch, 1) != 1) {
+	if (DgramWrite(sock, &ch, 1) != 1) {
 	    GetSocketError(sock);
-	    write(sock, &ch, 1);
+	    DgramWrite(sock, &ch, 1);
 	}
 	micro_delay((unsigned)50*1000);
     }
@@ -605,16 +603,16 @@ void Net_cleanup(void)
     }
     if (sock > 2) {
 	ch = PKT_QUIT;
-	if (write(sock, &ch, 1) != 1) {
+	if (DgramWrite(sock, &ch, 1) != 1) {
 	    GetSocketError(sock);
-	    write(sock, &ch, 1);
+	    DgramWrite(sock, &ch, 1);
 	}
 	micro_delay((unsigned)50*1000);
-	if (write(sock, &ch, 1) != 1) {
+	if (DgramWrite(sock, &ch, 1) != 1) {
 	    GetSocketError(sock);
-	    write(sock, &ch, 1);
+	    DgramWrite(sock, &ch, 1);
 	}
-	close(sock);
+	DgramClose(sock);
     }
 }
 
@@ -816,6 +814,7 @@ void Net_init_measurement(void)
 static int Net_packet(void)
 {
     int		type,
+		prev_type = 0,
 		result,
 		replyto,
 		status;
@@ -824,14 +823,15 @@ static int Net_packet(void)
 	type = (*rbuf.ptr & 0xFF);
 	if (receive_tbl[type] == NULL) {
 	    errno = 0;
-	    error("Received unknown packet type (%d)", type);
-	    return -1;
+	    error("Received unknown packet type (%d, %d), dropping frame.", type, prev_type);
+	    Sockbuf_clear(&rbuf);
+	    break;
 	}
 	else if ((result = (*receive_tbl[type])()) <= 0) {
 	    if (result == -1) {
 		if (type != PKT_QUIT) {
 		    errno = 0;
-		    error("Processing packet type (%d) failed", type);
+		    error("Processing packet type (%d, %d) failed", type, prev_type);
 		}
 		return -1;
 	    }
@@ -839,6 +839,7 @@ static int Net_packet(void)
 	    Sockbuf_clear(&rbuf);
 	    break;
 	}
+	prev_type = type;
     }
     while (cbuf.buf + cbuf.len > cbuf.ptr) {
 	type = (*cbuf.ptr & 0xFF);
@@ -1978,10 +1979,28 @@ int Receive_string(void)
     return 1;
 }
 
+int Receive_loseitem(void)
+{
+    int		n;
+    u_byte	pkt;
+				/* Most of the Receive_ funcs call a */
+				/* Handle_ func but that seems */
+				/* unecessary here */
+    if ((n = Packet_scanf(&rbuf, "%c%c", &pkt, &lose_item)) <= 0) {
+	return n;
+    }
+    return 1;
+}
+
 int Send_ack(long rel_loops)
 {
-    if (Packet_printf(&wbuf, "%c%ld%ld", PKT_ACK,
-		      reliable_offset, rel_loops) <= 0) {
+    int			n;
+
+    if ((n = Packet_printf(&wbuf, "%c%ld%ld", PKT_ACK,
+			   reliable_offset, rel_loops)) <= 0) {
+	if (n == 0) {
+	    return 0;
+	}
 	error("Can't ack reliable data");
 	return -1;
     }
@@ -2003,7 +2022,7 @@ int Receive_reliable(void)
     if (n == 0) {
 	errno = 0;
 	error("Incomplete reliable data packet");
-	return -1;
+	return 0;
     }
 #if DEBUG
     if (reliable_offset >= rel + len) {
@@ -2096,9 +2115,8 @@ int Send_keyboard(u_byte *keyboard_vector)
     const int	size = KEYBOARD_SIZE;
 
     if (wbuf.size - wbuf.len < size + 1 + 4) {
-	errno = 0;
-	error("Not enough write buffer space for keyboard state");
-	return -1;
+	/* Not enough write buffer space for keyboard state */
+	return 0;
     }
     Packet_printf(&wbuf, "%c%ld", PKT_KEYBOARD, last_keyboard_change);
     memcpy(&wbuf.buf[wbuf.len], keyboard_vector, size);
