@@ -1,4 +1,4 @@
-/* $Id: pack.h,v 3.42 1995/03/04 19:24:51 bert Exp $
+/* $Id: pack.h,v 3.49 1996/05/13 12:16:51 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -24,14 +24,15 @@
 #ifndef	PACK_H
 #define	PACK_H
 
-#define CAP_LETTER(c)	(c = (c>='a' && c<='z') ? c-'a'+'A' : c)
+#define CAP_LETTER(c)	((c) = ((c)>='a' && (c)<='z') ? (c)-'a'+'A' : (c))
 
 #define SERVER_PORT	15345		/* Port which server listens to. */
 #define SERVER_PORT_STR	"15345"		/* ASCII version of server port. */
 #define META_PORT	5500
 #define META_HOST	"xpilot.cs.uit.no"
-#define META_HOST_TWO	"xpilot.mc.bio.uva.nl"
+#define META_HOST_TWO	"prince.bio.uva.nl"
 #define META_IP		"129.242.16.101"
+#define META_IP_TWO	"145.18.160.57"
 
 /*
  * Magic contact word.
@@ -87,8 +88,11 @@
  * 3.2.6.1: New player timing packet.
  * 3.2.8.0: New asyn packet.
  * 3.3.1.0: Different owner-only commands.
+ * 3.3.2.0: Map decorations.
+ * 3.4.0.0: Lose/drop item key.
+ * 3.5.0.0: Player waiting queue.
  */
-#define	MAGIC		0x3310F4ED
+#define	MAGIC		0x3500F4ED
 
 #define MAGIC2VERSION(M)	(((M) >> 16) & 0xFFFF)
 #define VERSION2MAGIC(V)	((((V) & 0xFFFF) << 16) | (MAGIC & 0xFFFF))
@@ -114,6 +118,7 @@
  * Different contact pack types.
  */
 #define	ENTER_GAME_pack		0x00
+#define	ENTER_QUEUE_pack	0x01
 #define	REPLY_pack		0x10
 #define	REPORT_STATUS_pack	0x21
 #define	OPTION_LIST_pack	0x28
