@@ -1,4 +1,4 @@
-/* $Id: player.c,v 3.81 1995/01/11 19:50:13 bert Exp $
+/* $Id: player.c,v 3.95 1996/04/10 06:44:59 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -22,6 +22,7 @@
  */
 
 #include <stdlib.h>
+#include <string.h>
 #include <stdio.h>
 
 #define SERVER
@@ -35,12 +36,13 @@
 #include "bit.h"
 #include "netserver.h"
 #include "saudio.h"
+#include "error.h"
 
 char player_version[] = VERSION;
 
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: player.c,v 3.81 1995/01/11 19:50:13 bert Exp $";
+    "@(#)$Id: player.c,v 3.95 1996/04/10 06:44:59 bert Exp $";
 #endif
 
 extern int Rate(int winner, int loser);
@@ -177,8 +179,8 @@ void Go_home(int ind)
     pl->turnacc = pl->turnvel = 0.0;
     memset(pl->last_keyv, 0, sizeof(pl->last_keyv));
     memset(pl->prev_keyv, 0, sizeof(pl->prev_keyv));
-    pl->key_changed = 0;
     pl->used &= ~USED_KILL;
+
     if (playerStartsShielded != 0) {
 	SET_BIT(pl->used, OBJ_SHIELD);
 	if (playerShielding == 0) {
@@ -373,6 +375,7 @@ void Init_player(int ind, wireobj *ship)
     pl->shot_max	= ShotsMax;
     pl->shot_life	= ShotsLife;
     pl->shot_mass	= ShotsMass;
+    pl->shot_time	= 0;
     pl->color		= WHITE;
     pl->score		= 0;
     pl->prev_score	= 0;
@@ -459,6 +462,9 @@ void Init_player(int ind, wireobj *ship)
     pl->conn		= NOT_CONNECTED;
     pl->audio		= NULL;
 
+    pl->lose_item	= 0;
+    pl->lose_item_state	= 0;
+
     pl->shove_next = 0;
     for (i = 0; i < MAX_RECORDED_SHOVES; i++) {
 	pl->shove_record[i].pusher_id = -1;
@@ -626,7 +632,7 @@ static void Reset_all_players(void)
 		    World.targets[i].damage = TARGET_DAMAGE;
 		    World.targets[i].conn_mask = 0;
 		    World.targets[i].update_mask = (unsigned)-1;
-		    World.targets[i].last_change = loops;
+		    World.targets[i].last_change = frame_loops;
 		}
 	    }
 	}
@@ -1025,7 +1031,7 @@ static void Race_game_over(void)
 	free(order);
     }
 
-    for (i = 0; i < NumPlayers; i++)  {
+    for (i = NumPlayers - 1; i >= 0; i--)  {
 	pl = Players[i];
 	CLR_BIT(pl->status, RACE_OVER | FINISH);
 	if (BIT(pl->status, PAUSE)
@@ -1514,6 +1520,7 @@ void Delete_player(int ind)
 			OBJ_MINE|OBJ_SMART_SHOT|OBJ_HEAT_SHOT|OBJ_TORPEDO)) {
 		    obj->mass = 0;
 		}
+		obj->id = -1;
 	    }
 	}
 	else if (obj->owner == id) {
@@ -1548,20 +1555,35 @@ void Delete_player(int ind)
 
     /*
      * Swap entry no 'ind' with the last one.
+     *
+     * Change the Players[] pointer array to have Players[ind] point to
+     * a valid player and move our leaving player to Players[NumPlayers].
      */
     pl			= Players[NumPlayers];	/* Swap pointers... */
     Players[NumPlayers]	= Players[ind];
     Players[ind]	= pl;
-
-    Check_team_members (Players[ind]->team);
+    pl			= Players[NumPlayers];	/* Restore pointer. */
 
     GetInd[Players[ind]->id] = ind;
     GetInd[Players[NumPlayers]->id] = NumPlayers;
 
-    for (i=0; i<NumPlayers; i++) {
-	if (BIT(Players[i]->lock.tagged, LOCK_PLAYER)
-	    && (Players[i]->lock.pl_id == id || NumPlayers <= 1))
-	    CLR_BIT(Players[i]->lock.tagged, LOCK_PLAYER);
+    Check_team_members(pl->team);
+
+    for (i = NumPlayers - 1; i >= 0; i--) {
+	if (IS_TANK_IND(i)
+	    && Players[i]->lock.pl_id == id) {
+	    /* remove tanks which were released by this player. */
+	    Delete_player(i);
+	    continue;
+	}
+	if (BIT(Players[i]->lock.tagged, LOCK_PLAYER|LOCK_VISIBLE)
+	    && (Players[i]->lock.pl_id == id || NumPlayers <= 1)) {
+	    CLR_BIT(Players[i]->lock.tagged, LOCK_PLAYER|LOCK_VISIBLE);
+	    /* bugfix (1995/08/07): */
+	    Players[i]->tractor = NULL;
+	    /* consider this: */
+	    CLR_BIT(Players[i]->used, OBJ_TRACTOR_BEAM);
+	}
 	if (IS_ROBOT_IND(i)
 	    && Robot_war_on_player(i) == id) {
 	    Robot_reset_war(i);
@@ -1577,10 +1599,15 @@ void Delete_player(int ind)
 	}
     }
 
-    for (i = 0; i < NumPlayers; i++) {
+    for (i = NumPlayers - 1; i >= 0; i--) {
 	if (Players[i]->conn != NOT_CONNECTED) {
 	    Send_leave(Players[i]->conn, id);
 	}
+	else if (IS_TANK_IND(i)) {
+	    if (Players[i]->lock.pl_id == id) {
+		Delete_player(i);
+	    }
+	}
     }
 
 }
@@ -1607,6 +1634,9 @@ void Detach_ball(int ind, int obj)
 	}
 	if (cnt == 0)
 	    CLR_BIT(Players[ind]->have, OBJ_BALL);
+	else {
+	    sound_play_sensors(Players[ind]->pos.x, Players[ind]->pos.y, DROP_BALL_SOUND);
+	}
     }
 }
 
@@ -1625,7 +1655,6 @@ void Player_death_reset(int ind)
 
     if (IS_TANK_PTR(pl)) {
 	Delete_player(ind);
-	updateScores = true;
 	return;
     }
 
@@ -1683,6 +1712,7 @@ void Player_death_reset(int ind)
     if (pl->life == -1) {
 	pl->life = 0;
 	SET_BIT(pl->status, GAME_OVER);
+	Player_lock_closest(ind, 0);
 	pl->mychar = 'D';
     }
 
