@@ -1,4 +1,4 @@
-/* $Id: record.c,v 3.8 1995/03/06 10:37:25 bert Exp $
+/* $Id: record.c,v 3.10 1996/05/02 16:05:56 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -21,12 +21,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifdef VMS
-#include <unixio.h>
-#include <unixlib.h>
-#else
 #include <unistd.h>
-#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -683,7 +678,7 @@ static int RFillRectangles(Display *display, Drawable drawable, GC gc,
 	putc(RC_FILLRECTANGLES, recordFP);
 	RWriteGC(gc, GCForeground | RTILEGC);
 	RWriteUShort(nrectangles);
-	for(i = 0; i < nrectangles; i++) {
+	for (i = 0; i < nrectangles; i++) {
 	    RWriteShort(rectangles[i].x);
 	    RWriteShort(rectangles[i].y);
 	    RWriteByte(rectangles[i].width);
