@@ -1,4 +1,4 @@
-/* $Id: texture.h,v 3.1 1995/01/24 17:24:02 bert Exp $
+/* $Id: texture.h,v 3.3 1996/05/04 21:43:49 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -27,5 +27,7 @@
  * Load a texture into a pixmap.
  */
 extern Pixmap Texture_wall(void);
+extern Pixmap Texture_decor(void);
+extern Pixmap Texture_ball(void);
 
 #endif
