@@ -1,4 +1,4 @@
-/* $Id: update.c,v 3.63 1995/01/11 20:00:31 bert Exp $
+/* $Id: update.c,v 3.70 1996/04/07 17:05:18 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -39,7 +39,7 @@ char update_version[] = VERSION;
 
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: update.c,v 3.63 1995/01/11 20:00:31 bert Exp $";
+    "@(#)$Id: update.c,v 3.70 1996/04/07 17:05:18 bert Exp $";
 #endif
 
 
@@ -73,17 +73,17 @@ static void Transport_to_home(int ind)
     const int		T = RECOVERY_DELAY;
 
     if (BIT(World.rules->mode, TIMING) && pl->round) {
-	    int check;
+	int check;
 
-	    if (pl->check)
-		    check = pl->check - 1;
-	    else
-		    check = World.NumChecks - 1;
-	    bx = World.check[check].x * BLOCK_SZ + BLOCK_SZ/2;
-	    by = World.check[check].y * BLOCK_SZ + BLOCK_SZ/2;
+	if (pl->check)
+		check = pl->check - 1;
+	else
+		check = World.NumChecks - 1;
+	bx = World.check[check].x * BLOCK_SZ + BLOCK_SZ/2;
+	by = World.check[check].y * BLOCK_SZ + BLOCK_SZ/2;
     } else {
-	    bx = World.base[pl->home_base].pos.x * BLOCK_SZ + BLOCK_SZ/2;
-	    by = World.base[pl->home_base].pos.y * BLOCK_SZ + BLOCK_SZ/2;
+	bx = World.base[pl->home_base].pos.x * BLOCK_SZ + BLOCK_SZ/2;
+	by = World.base[pl->home_base].pos.y * BLOCK_SZ + BLOCK_SZ/2;
     }
     dx = WRAP_DX(bx - pl->pos.x);
     dy = WRAP_DY(by - pl->pos.y);
@@ -350,7 +350,7 @@ static void do_Autopilot (player *pl)
  * Do tractor beam attraction between two players, where `pl' is doing
  * the tractor beam and `to' is the target.
  */
-void do_Tractor_beam (player *pl)
+static void Tractor_beam (player *pl)
 {
     player		*victim;
     float		maxdist, maxforce, percent;
@@ -397,13 +397,13 @@ void do_Tractor_beam (player *pl)
     dvy = tsin(theta) * (force / pl->mass);
     pl->vel.x += dvx;
     pl->vel.y += dvy;
-    Record_shove(pl, victim, loops);
+    Record_shove(pl, victim, frame_loops);
 
     dvx = -(tcos(theta) * (force / victim->mass));
     dvy = -(tsin(theta) * (force / victim->mass));
     victim->vel.x += dvx;
     victim->vel.y += dvy;
-    Record_shove(victim, pl, loops);
+    Record_shove(victim, pl, frame_loops);
 }
 
 /********** **********
@@ -428,8 +428,7 @@ void Update_objects(void)
     if (fireRepeatRate > 0) {
 	for (i = 0; i < NumPlayers; i++) {
 	    pl = Players[i];
-	    if (BIT(pl->used, OBJ_SHOT)
-		&& loops - pl->shot_time >= fireRepeatRate) {
+	    if (BIT(pl->used, OBJ_SHOT)) {
 		Fire_normal_shots(i);
 	    }
 	}
@@ -459,7 +458,7 @@ void Update_objects(void)
 	    if ((World.fuel[i].fuel += fuel) >= MAX_STATION_FUEL) {
 		World.fuel[i].fuel = MAX_STATION_FUEL;
 	    }
-	    else if (World.fuel[i].last_change + frames_per_update > loops) {
+	    else if (World.fuel[i].last_change + frames_per_update > frame_loops) {
 		/*
 		 * We don't send fuelstation info to the clients every frame
 		 * if it wouldn't change their display.
@@ -467,7 +466,7 @@ void Update_objects(void)
 		continue;
 	    }
 	    World.fuel[i].conn_mask = 0;
-	    World.fuel[i].last_change = loops;
+	    World.fuel[i].last_change = frame_loops;
 	}
     }
 
@@ -501,7 +500,7 @@ void Update_objects(void)
 		World.block[World.cannon[i].pos.x][World.cannon[i].pos.y]
 		    = CANNON;
 		World.cannon[i].conn_mask = 0;
-		World.cannon[i].last_change = loops;
+		World.cannon[i].last_change = frame_loops;
 	    }
 	    continue;
 	}
@@ -522,7 +521,7 @@ void Update_objects(void)
 		    = TARGET;
 		World.targets[i].conn_mask = 0;
 		World.targets[i].update_mask = (unsigned)-1;
-		World.targets[i].last_change = loops;
+		World.targets[i].last_change = frame_loops;
 
 		if (targetSync) {
 		    u_short team = World.targets[i].team;
@@ -533,7 +532,7 @@ void Update_objects(void)
 				       [World.targets[j].pos.y] = TARGET;
 			    World.targets[j].conn_mask = 0;
 			    World.targets[j].update_mask = (unsigned)-1;
-			    World.targets[j].last_change = loops;
+			    World.targets[j].last_change = frame_loops;
 			    World.targets[j].dead_time = 0;
 			    World.targets[j].damage = TARGET_DAMAGE;
 			}
@@ -549,7 +548,7 @@ void Update_objects(void)
 	if (World.targets[i].damage >= TARGET_DAMAGE) {
 	    World.targets[i].damage = TARGET_DAMAGE;
 	}
-	else if (World.targets[i].last_change + TARGET_UPDATE_DELAY < loops) {
+	else if (World.targets[i].last_change + TARGET_UPDATE_DELAY < frame_loops) {
 	    /*
 	     * We don't send target info to the clients every frame
 	     * if the latest repair wouldn't change their display.
@@ -557,7 +556,7 @@ void Update_objects(void)
 	    continue;
 	}
 	World.targets[i].conn_mask = 0;
-	World.targets[i].last_change = loops;
+	World.targets[i].last_change = frame_loops;
     }
 
     /* * * * * *
@@ -595,6 +594,7 @@ void Update_objects(void)
 		Go_home(i);
 	    }
 	    if (BIT(pl->status, SELF_DESTRUCT)) {
+		SET_BIT(pl->status, KILLED);
 		sprintf(msg, "%s has comitted suicide.", pl->name);
 		Set_message(msg);
 		Throw_items(pl);
@@ -613,7 +613,8 @@ void Update_objects(void)
 		}
 	    }
 	    if (BIT(pl->used, OBJ_SHIELD) == 0) {
-		if (!BIT(pl->have, OBJ_EMERGENCY_SHIELD)) {
+		/* BG 95/06/03: change test on "have" to "used". */
+		if (!BIT(pl->used, OBJ_EMERGENCY_SHIELD)) {
 		    CLR_BIT(pl->have, OBJ_SHIELD);
 		}
 		pl->shield_time = 0;
@@ -693,9 +694,9 @@ void Update_objects(void)
 	    else if (pl->updateVisibility
 		     || Players[j]->updateVisibility
 		     || rand() % UPDATE_RATE
-		     < ABS(loops - pl->visibility[j].lastChange)) {
+		     < ABS(frame_loops - pl->visibility[j].lastChange)) {
 
-		pl->visibility[j].lastChange = loops;
+		pl->visibility[j].lastChange = frame_loops;
 		pl->visibility[j].canSee
 		    = rand() % (pl->item[ITEM_SENSOR] + 1)
 			> (rand() % (Players[j]->item[ITEM_CLOAK] + 1));
@@ -703,11 +704,11 @@ void Update_objects(void)
 	}
 
 	if (BIT(pl->used, OBJ_REFUEL)) {
-	    if ((Wrap_length((pl->pos.x-World.fuel[pl->fs].pix_pos.x),
-			     (pl->pos.y-World.fuel[pl->fs].pix_pos.y)) > 90.0)
+	    if ((Wrap_length(pl->pos.x - World.fuel[pl->fs].pix_pos.x,
+			     pl->pos.y - World.fuel[pl->fs].pix_pos.y) > 90.0)
 		|| (pl->fuel.sum >= pl->fuel.max)
-		|| (World.block[World.fuel[i].blk_pos.x]
-			       [World.fuel[i].blk_pos.y] != FUEL)) {
+		|| (World.block[World.fuel[pl->fs].blk_pos.x]
+			       [World.fuel[pl->fs].blk_pos.y] != FUEL)) {
 		CLR_BIT(pl->used, OBJ_REFUEL);
 	    } else {
 		int i = pl->fuel.num_tanks;
@@ -717,13 +718,13 @@ void Update_objects(void)
 		    if (World.fuel[pl->fs].fuel > REFUEL_RATE) {
 			World.fuel[pl->fs].fuel -= REFUEL_RATE;
 			World.fuel[pl->fs].conn_mask = 0;
-			World.fuel[pl->fs].last_change = loops;
+			World.fuel[pl->fs].last_change = frame_loops;
 			Add_fuel(&(pl->fuel), REFUEL_RATE);
 		    } else {
 			Add_fuel(&(pl->fuel), World.fuel[pl->fs].fuel);
 			World.fuel[pl->fs].fuel = 0;
 			World.fuel[pl->fs].conn_mask = 0;
-			World.fuel[pl->fs].last_change = loops;
+			World.fuel[pl->fs].last_change = frame_loops;
 			CLR_BIT(pl->used, OBJ_REFUEL);
 			break;
 		    }
@@ -753,7 +754,7 @@ void Update_objects(void)
 		    if (pl->fuel.tank[pl->fuel.current] > REFUEL_RATE) {
 			targ->damage += TARGET_FUEL_REPAIR_PER_FRAME;
 			targ->conn_mask = 0;
-			targ->last_change = loops;
+			targ->last_change = frame_loops;
 			Add_fuel(&(pl->fuel), -REFUEL_RATE);
 			if (targ->damage > TARGET_DAMAGE) {
 			    targ->damage = TARGET_DAMAGE;
@@ -976,7 +977,7 @@ void Update_objects(void)
 	}
 
 	if (BIT(pl->used, OBJ_TRACTOR_BEAM))
-	    do_Tractor_beam (pl);
+	    Tractor_beam(pl);
 	else
 	    pl->tractor = NULL;
     }
@@ -1022,7 +1023,7 @@ void Update_objects(void)
     /*
      * Now update labels if need be.
      */
-    if (updateScores && loops % UPDATE_SCORE_DELAY == 0)
+    if (updateScores && frame_loops % UPDATE_SCORE_DELAY == 0)
 	Update_score_table();
 }
 
