@@ -1,4 +1,4 @@
-/* $Id: xinit.c,v 3.86 1995/02/04 15:03:46 bert Exp $
+/* $Id: xinit.c,v 3.93 1996/04/27 18:39:08 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -74,7 +74,7 @@ char xinit_version[] = VERSION;
 
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: xinit.c,v 3.86 1995/02/04 15:03:46 bert Exp $";
+    "@(#)$Id: xinit.c,v 3.93 1996/04/27 18:39:08 bert Exp $";
 #endif
 
 /* Kludge for visuals under C++ */
@@ -118,6 +118,7 @@ Visual			*visual;
 int			dispDepth;
 bool			mono;
 bool			colorSwitch;
+bool			multibuffer;
 int			top_width, top_height, top_x, top_y, top_posmask;
 int			draw_width, draw_height;
 char			*geometry;
@@ -377,9 +378,11 @@ static void Fill_colormap(void)
     /* Check for misunderstanding of X colormap stuff. */
     for (i = 0; i < max_fill; i++) {
 	if (pixels[i] != i) {
+#ifdef DEVELOPMENT
 	    errno = 0;
 	    error("Can't pre-fill color map, got %d'th pixel %lu",
 		  i, pixels[i]);
+#endif
 	    XFreeColors(dpy, colormap, pixels, max_fill, 0);
 	    return;
 	}
@@ -498,6 +501,7 @@ static void Get_visual_info(void)
 	visual = DefaultVisual(dpy, DefaultScreen(dpy));
 	if (visual->class == TrueColor || visual->class == DirectColor) {
 	    visual_class = PseudoColor;
+	    strcpy(visualName, "PseudoColor");
 	}
     }
     if (visual_class >= 0 || visual_id >= 0) {
@@ -676,11 +680,11 @@ void Init_disp_prop(Display *d, Window win, int w, int h, int x, int y,
 }
 
 /*
- * The following function initializes a player window.
+ * The following function initializes a toplevel window.
  * It returns 0 if the initialization was successful,
  * or -1 if it couldn't initialize the double buffering routine.
  */
-int Init_window(void)
+int Init_top(void)
 {
     int				i, p, values,
 				num_planes,
@@ -689,12 +693,8 @@ int Init_window(void)
 				x, y;
     unsigned			w, h;
     XGCValues			xgc;
-    int				button_form,
-				menu_button;
     XSetWindowAttributes	sattr;
     unsigned long		mask;
-    Pixmap			pix;
-    GC				cursorGC;
 
     colormap = 0;
 
@@ -770,15 +770,28 @@ int Init_window(void)
     /*
      * Initialize the double buffering routine.
      */
-    dbuf_state = start_dbuff(dpy,
-			     (colormap != 0)
-				 ? colormap
-				 : DefaultColormap(dpy,
-						   DefaultScreen(dpy)),
-			     (colorSwitch) ? COLOR_SWITCH : PIXMAP_COPY,
-			     num_planes,
-			     colors);
-
+    dbuf_state = NULL;
+    if (multibuffer) {
+	dbuf_state = start_dbuff(dpy,
+				 (colormap != 0)
+				     ? colormap
+				     : DefaultColormap(dpy,
+						       DefaultScreen(dpy)),
+				 MULTIBUFFER,
+				 num_planes,
+				 colors);
+    }
+    if (dbuf_state == NULL) {
+	multibuffer = false;
+	dbuf_state = start_dbuff(dpy,
+				 (colormap != 0)
+				     ? colormap
+				     : DefaultColormap(dpy,
+						       DefaultScreen(dpy)),
+				 ((colorSwitch) ? COLOR_SWITCH : PIXMAP_COPY),
+				 num_planes,
+				 colors);
+    }
     if (dbuf_state == NULL && colormap == 0) {
 
 	/*
@@ -791,7 +804,7 @@ int Init_window(void)
 	 * Try to initialize the double buffering again.
 	 */
 	dbuf_state = start_dbuff(dpy, colormap,
-				 (colorSwitch) ? COLOR_SWITCH : PIXMAP_COPY,
+				 ((colorSwitch) ? COLOR_SWITCH : PIXMAP_COPY),
 				 num_planes,
 				 colors);
     }
@@ -800,7 +813,7 @@ int Init_window(void)
 	/* Can't setup double buffering */
 	errno = 0;
 	error("Can't setup colors with visual %s and %d colormap entries",
-	    Visual_class_name(visual->class), visual->map_entries);
+	      Visual_class_name(visual->class), visual->map_entries);
 	return -1;
     }
 
@@ -816,20 +829,21 @@ int Init_window(void)
     if (wallColor >= maxColors || wallColor <= 0) {
 	wallColor = BLUE;
     }
+    if (wallRadarColor >= maxColors
+	|| (wallRadarColor & 1)) {
+	wallRadarColor = BLUE;
+    }
     if (targetRadarColor >= maxColors
-	|| (targetRadarColor != 8
-	    && targetRadarColor != 4
-	    && targetRadarColor != 2)) {
+	|| (targetRadarColor & 1)) {
 	targetRadarColor = BLUE;
     }
-
-
-    about_created = false;
-    talk_created = false;
-    talk_mapped = false;
-    talk_str[0] = '\0';
-    talk_cursor.point = 0;
-
+    if (decorColor >= maxColors || decorColor <= 0) {
+	decorColor = RED;
+    }
+    if (decorRadarColor >= maxColors
+	|| (decorRadarColor & 1)) {
+	decorRadarColor = 2;
+    }
 
     /*
      * Get toplevel geometry.
@@ -883,7 +897,6 @@ int Init_window(void)
 	geometry = NULL;
     }
 
-
     /*
      * Create toplevel window (we need this first so that we can create GCs)
      */
@@ -903,6 +916,10 @@ int Init_window(void)
 			0, dispDepth,
 			InputOutput, visual,
 			mask, &sattr);
+    XSelectInput(dpy, top,
+		 KeyPressMask | KeyReleaseMask
+		 | FocusChangeMask | StructureNotifyMask);
+    Init_disp_prop(dpy, top, top_width, top_height, top_x, top_y, top_flags);
     if (kdpy) {
 	int scr = DefaultScreen(kdpy);
 	keyboard = XCreateSimpleWindow(kdpy,
@@ -910,6 +927,10 @@ int Init_window(void)
 				       top_x, top_y,
 				       top_width, top_height,
 				       0, 0, BlackPixel(dpy, scr));
+	XSelectInput(kdpy, keyboard,
+		     KeyPressMask | KeyReleaseMask | FocusChangeMask);
+	Init_disp_prop(kdpy, keyboard, top_width, top_height,
+		       top_x, top_y, top_flags);
     }
 
     /*
@@ -921,7 +942,6 @@ int Init_window(void)
 				    (char *)itemBitmapData[i].data,
 				    ITEM_SIZE, ITEM_SIZE);
 
-
     /*
      * Creates and initializes the graphic contexts.
      */
@@ -952,7 +972,6 @@ int Init_window(void)
     XSetBackground(dpy, gc, colors[BLACK].pixel);
     XSetDashes(dpy, gc, 0, dashes, NUM_DASHES);
 
-
     /*
      * Set fonts
      */
@@ -992,11 +1011,9 @@ int Init_window(void)
 	      BlackPixel(dpy, DefaultScreen(dpy)),
 	      GXcopy, AllPlanes);
 
-
-    if (colorSwitch)
+    if (dbuf_state->type == COLOR_SWITCH)
 	XSetPlaneMask(dpy, gc, dbuf_state->drawing_planes);
 
-
     /*
      * A little hack that enables us to draw on both sets of double buffering
      * planes at once.
@@ -1012,10 +1029,6 @@ int Init_window(void)
 		    dpl_1[p] |= 1<<i;
     }
 
-
-    /*
-     * Creates the windows.
-     */
     if (mono) {
 	buttonColor = BLACK;
 	windowColor = BLACK;
@@ -1026,6 +1039,27 @@ int Init_window(void)
 	borderColor = WHITE;
     }
 
+    return 0;
+}
+
+/*
+ * Creates the playing windows.
+ * Returns 0 on success, -1 on error.
+ */
+int Init_windows(void)
+{
+    unsigned			w, h;
+    int				button_form,
+				menu_button;
+    Pixmap			pix;
+    GC				cursorGC;
+
+    if (!top) {
+	if (Init_top()) {
+	    return -1;
+	}
+    }
+
     draw_width = top_width - (256 + 2);
     draw_height = top_height;
     draw = XCreateSimpleWindow(dpy, top, 258, 0,
@@ -1083,39 +1117,40 @@ int Init_window(void)
     /*
      * Selecting the events we can handle.
      */
-    XSelectInput(dpy, top,
-		 KeyPressMask | KeyReleaseMask
-		 | FocusChangeMask | StructureNotifyMask);
-    if (kdpy)
-	XSelectInput(kdpy, keyboard,
-		     KeyPressMask | KeyReleaseMask | FocusChangeMask);
     XSelectInput(dpy, radar, ExposureMask);
     XSelectInput(dpy, players, ExposureMask);
     XSelectInput(dpy, draw, 0);
 
-    Init_disp_prop(dpy, top, top_width, top_height, top_x, top_y, top_flags);
-    if (kdpy)
-	Init_disp_prop(kdpy, keyboard, top_width, top_height,
-		       top_x, top_y, top_flags);
 
     /*
      * Initialize misc. pixmaps if we're not color switching.
+     * (This could be in dbuff_init completely IMHO, -- Metalite)
      */
-    if (colorSwitch == false) {
+    switch (dbuf_state->type) {
+
+    case PIXMAP_COPY:
 	p_radar = XCreatePixmap(dpy, radar, 256, RadarHeight, dispDepth);
 	s_radar = XCreatePixmap(dpy, radar, 256, RadarHeight, dispDepth);
 	p_draw  = XCreatePixmap(dpy, draw, draw_width, draw_height, dispDepth);
-    }
-    else {
+	break;
+
+    case MULTIBUFFER:
+	p_radar = XCreatePixmap(dpy, radar, 256, RadarHeight, dispDepth);
+	s_radar = XCreatePixmap(dpy, radar, 256, RadarHeight, dispDepth);
+	dbuff_init(dbuf_state);
+	break;
+
+    case COLOR_SWITCH:
 	s_radar = radar;
 	p_radar = s_radar;
 	p_draw = draw;
 	Paint_sliding_radar();
+	break;
     }
 
     XAutoRepeatOff(dpy);	/* We don't want any autofire, yet! */
     if (kdpy)
-	    XAutoRepeatOff(kdpy);
+	XAutoRepeatOff(kdpy);
 
     /*
      * Define a blank cursor for use with pointer control
@@ -1144,6 +1179,12 @@ int Init_window(void)
 
     Init_spark_colors();
 
+    about_created = false;
+    talk_created = false;
+    talk_mapped = false;
+    talk_str[0] = '\0';
+    talk_cursor.point = 0;
+
     return 0;
 }
 
@@ -1744,6 +1785,7 @@ void Talk_map_window(bool map)
 		     0, 0, 0, 0,
 		     TALK_WINDOW_WIDTH - 2,
 		     TALK_WINDOW_HEIGHT - 2);
+	XFlush(dpy);	/* warp pointer ASAP. */
     }
     else if (talk_created == true) {
 	XUnmapWindow(dpy, talk_w);
@@ -2044,7 +2086,7 @@ void Resize(Window w, int width, int height)
     Send_display();
     Net_flush();
     XResizeWindow(dpy, draw, draw_width, draw_height);
-    if (p_draw != draw) {
+    if (dbuf_state->type == PIXMAP_COPY) {
 	XFreePixmap(dpy, p_draw);
 	p_draw = XCreatePixmap(dpy, draw, draw_width, draw_height, dispDepth);
     }
