@@ -1,4 +1,4 @@
-/* $Id: xinit.h,v 3.35 1995/01/29 16:22:23 bert Exp $
+/* $Id: xinit.h,v 3.37 1996/04/27 18:39:09 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -47,6 +47,7 @@ extern Visual		*visual;
 extern int		dispDepth;
 extern bool		mono;
 extern bool		colorSwitch;
+extern bool		multibuffer;
 extern char		color_names[MAX_COLORS][MAX_COLOR_LEN];
 extern int		top_width, top_height;
 extern int		draw_width, draw_height;
@@ -61,7 +62,8 @@ extern int		num_spark_colors;
  */
 extern int Parse_colors(Colormap cmap);
 extern void List_visuals(void);
-extern int Init_window(void);
+extern int Init_top(void);
+extern int Init_windows(void);
 extern int Alloc_msgs(void);
 extern void Free_msgs(void);
 extern void Expose_info_window(void);
