@@ -1,4 +1,4 @@
-/* $Id: xp-replay.c,v 3.20 1995/03/06 12:09:52 bert Exp $
+/* $Id: xp-replay.c,v 3.25 1996/05/02 16:06:03 bert Exp $
  *
  * XP-Replay, playback an XPilot session.  Copyright (C) 1994-95 by
  *
@@ -34,6 +34,9 @@
 ** implied warranty.
  */
 
+#ifdef VMS
+#include <socket.h>
+#endif
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -53,6 +56,10 @@
 					(struct process_stats *)NULL, \
 					(struct process_stats *)NULL)
 #endif
+#if !defined(select) && defined(__linux__)
+#define select(N, R, W, E, T)   select((N),             \
+        (fd_set*)(R), (fd_set*)(W), (fd_set*)(E), (T))
+#endif
 
 #include "recordfmt.h"
 #include "item.h"
@@ -524,6 +531,7 @@ void *MyMalloc(size_t size, enum MemTypes mt)
     return p;
 }
 
+#if 0
 /*
  * Wrapper for strdup(3).
  * This one uses MyMalloc() instead of malloc()
@@ -535,6 +543,7 @@ static char *MyStrDup(const char *str)
     memcpy(dup, str, size);
     return dup;
 }
+#endif
 
 /*
  * Read one 8-bit byte from the recorded input stream.
@@ -699,7 +708,6 @@ static Pixmap RReadTile(struct xprc *rc)
     unsigned			width, height;
     unsigned			x, y;
     unsigned			depth;
-    int				i;
     XImage			*img;
     unsigned char		ch;
     Pixmap			tile;
@@ -1999,7 +2007,7 @@ static void Init_topmain(struct xui *ui, struct xprc *rc)
 
     x = 5;
     y = (toph>>1) + 5;
-    for(i=0; i<NUM_BUTTONS; i++)
+    for (i=0; i<NUM_BUTTONS; i++)
     {
 	union button_image p;
 
@@ -2053,21 +2061,21 @@ static void Init_topmain(struct xui *ui, struct xprc *rc)
     mw = 0;
     ui->labels[0].x = 5;
     ui->labels[0].y = toph + 15;
-    for(i=1; i<5; i++)
+    for (i=1; i<5; i++)
     {
 	ui->labels[i].x = ui->labels[i-1].x;
 	ui->labels[i].y = ui->labels[i-1].y
 	    + ui->boldFont->ascent + ui->boldFont->descent + 2;
     }
     toph = ui->labels[4].y + ui->boldFont->ascent + ui->boldFont->descent + 5;
-    for(i=0; i<5; i++)
+    for (i=0; i<5; i++)
     {
 	w = XTextWidth(ui->boldFont, ui->labels[i].name,
 		       strlen(ui->labels[i].name));
 	if (w > mw)
 	    mw = w;
     }
-    for(i=0; i<5; i++)
+    for (i=0; i<5; i++)
 	ui->labels[i].w = mw;
     ui->labels[5].x = topw - ui->labels[0].x;
     ui->labels[5].y = ui->labels[0].y;
@@ -2788,10 +2796,6 @@ static void RWriteGC(struct xprc *rc, struct rGC *gcp, FILE *fp)
 
 static void WriteHeader(struct xprc *rc, FILE *fp)
 {
-    extern char			hostname[];
-    time_t			t;
-    char			buf[256];
-    char			*ptr;
     int				i;
 
     rewind(fp);
@@ -2835,7 +2839,6 @@ static void WriteFrame(struct xprc *rc, struct frame *f, FILE *fp)
 {
     /* drawShapes(save, pixmap, rc); */
     struct shape	*sp;
-    XGCValues		values;
     int			i;
 
     putc(RC_NEWFRAME, fp);
@@ -2992,7 +2995,6 @@ static void SaveFramesXPR(struct xprc *rc)
     struct frame	*begin = rc->save_first;
     struct frame	*end = rc->save_last;
     struct frame	*save;
-    int			i;
     int			done = 0;
     FILE		*fp;
     tile_list_t		*tptr;
@@ -3404,7 +3406,9 @@ int main(int argc, char **argv)
 	    verbose = 1;
 	}
 	else if (!strcmp(argv[argi], "-compress")) {
+#ifndef VMS
 	    compress = 1;
+#endif
 	}
 	else if (!strcmp(argv[argi], "-scale")) {
 	    if (++argi == argc || sscanf(argv[argi], "%lf", &scale) != 1) {
