@@ -1,4 +1,4 @@
-/* $Id: xpilot.c,v 3.61 1995/03/06 09:53:20 bert Exp $
+/* $Id: xpilot.c,v 3.74 1996/05/02 16:06:05 bert Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
@@ -24,39 +24,30 @@
 #ifdef _IBMESA
 # define _SOCKADDR_LEN
 #endif
-#ifdef VMS
-#include <unixio.h>
-#include <unixlib.h>
-#else
 #include <unistd.h>
-#endif
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
 #ifdef VMS
-#include "username.h"
-#include <socket.h>
-#include <in.h>
-#include <inet.h>
+# include "username.h"
+# include <socket.h>
+# include <in.h>
+# include <inet.h>
 #else
-#include <pwd.h>
-#include <sys/types.h>
-#include <sys/param.h>
-#include <sys/socket.h>
-#include <sys/ioctl.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+# include <pwd.h>
+# include <sys/types.h>
+# include <sys/param.h>
+# include <sys/socket.h>
+# include <sys/ioctl.h>
+# include <netinet/in.h>
+# include <arpa/inet.h>
 #endif
 #if defined(SVR4) || defined(__svr4__)
 # include <sys/sockio.h>
 #endif
-#ifdef VMS
-#include <time.h>
-#else
-#include <sys/time.h>
-#endif
+# include <sys/time.h>
 #ifndef LINUX0
-#include <net/if.h>
+# include <net/if.h>
 #endif
 #include <netdb.h>
 #include <string.h>
@@ -71,7 +62,7 @@
 #include "socklib.h"
 #include "net.h"
 #ifdef SUNCMW
-#include "cmw.h"
+# include "cmw.h"
 #endif /* SUNCMW */
 
 char xpilot_version[] = VERSION;
@@ -79,7 +70,7 @@ char xpilot_version[] = VERSION;
 #ifndef	lint
 static char versionid[] = "@(#)$" TITLE " $";
 static char sourceid[] =
-    "@(#)$Id: xpilot.c,v 3.61 1995/03/06 09:53:20 bert Exp $";
+    "@(#)$Id: xpilot.c,v 3.74 1996/05/02 16:06:05 bert Exp $";
 #endif
 
 #if defined(LINUX0) || defined(VMS)
@@ -119,6 +110,7 @@ static int		auto_connect = false,
 			auto_shutdown = false;
 static unsigned		server_version;
 static int		team = TEAM_NOT_SET;
+static int		onesock = 0;
 
 char			**Argv;
 int			Argc;
@@ -146,6 +138,47 @@ static void printfile(char *filename)
 }
 
 
+/* just like fgets() but strips newlines like gets() */
+static char* getline(char* buf, int len, FILE* stream)
+{
+	char* ret;
+	char* newline;
+
+	ret = fgets(buf, len, stream);
+	if (ret && (newline=strchr(ret, '\n'))) {
+		*newline = '\0';		/* strip newline */
+	}
+	return ret;
+}
+
+
+static int create_dgram_addr_socket(char *dotaddr, int port)
+{
+    static int save_fd = -1;
+    int fd;
+
+    if (save_fd == -1) {
+	fd = CreateDgramAddrSocket(dotaddr, port);
+	if (onesock) {
+	    save_fd = fd;
+	}
+    } else {
+	fd = save_fd;
+    }
+    return fd;
+}
+
+static int create_dgram_socket(int port)
+{
+    return create_dgram_addr_socket("0.0.0.0", port);
+}
+
+static void close_dgram_socket(int fd)
+{
+    if (!onesock) {
+	DgramClose(fd);
+    }
+}
 
 static bool Get_contact_message(sockbuf_t *sbuf)
 {
@@ -213,7 +246,7 @@ static int Get_reply_message(sockbuf_t *ibuf)
 
     if (SocketReadable(ibuf->sock)) {
 	Sockbuf_clear(ibuf);
-	if ((len = read(ibuf->sock, ibuf->buf, ibuf->size)) == -1) {
+	if ((len = DgramRead(ibuf->sock, ibuf->buf, ibuf->size)) == -1) {
 	    error("Can't read reply message from %s/%d",
 		  server_addr, server_port);
 	    exit(1);
@@ -282,12 +315,13 @@ static bool Process_commands(sockbuf_t *ibuf)
 {
     int			i, len, retries, delay, max_robots;
     char		c, status, reply_to, str[MAX_LINE];
-    unsigned short	port;
+    unsigned short	port, qpos;
     int			has_credentials = 0;
     int			cmd_credentials = 0;
     int			privileged_cmd;
     int			max_replies;
     long		key = 0;
+    time_t		qsent = 0;
     static char		localhost[] = "127.0.0.1";
 
 
@@ -309,7 +343,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 	else if (!auto_connect) {
 	    printf("*** Server on %s. Enter command> ", server_name);
 
-	    gets(str);
+	    getline(str, MAX_LINE-1, stdin);
 	    if (feof(stdin)) {
 		puts("");
 		c = 'Q';
@@ -326,6 +360,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 		c = 'D';
 	    else
 		c = 'J';
+	    str[0] = str[1] = '\0';
 	}
 
 	/*
@@ -334,35 +369,35 @@ static bool Process_commands(sockbuf_t *ibuf)
 	 * This assures us that we only get replies to the last command sent.
 	 */
 	if (ibuf->sock != -1) {
-	    SocketClose(ibuf->sock);
+	    close_dgram_socket(ibuf->sock);
 	    ibuf->sock = -1;
 	}
 
 	privileged_cmd = (strchr("DKLMOR", c) != NULL);
 	if (privileged_cmd) {
 	    if (!has_credentials) {
-		if ((ibuf->sock = CreateDgramAddrSocket(server_addr, 0)) == -1) {
+		if ((ibuf->sock = create_dgram_addr_socket(server_addr, 0)) == -1) {
 		    printf("Server %s is not local, privileged command not possible.\n",
 			   server_addr);
 		    continue;
 		}
-		close(ibuf->sock);
+		close_dgram_socket(ibuf->sock);
 	    }
-	    if ((ibuf->sock = CreateDgramAddrSocket(localhost, 0)) == -1) {
+	    if ((ibuf->sock = create_dgram_addr_socket(localhost, 0)) == -1) {
 		error("Could not create localhost socket");
 		exit(1);
 	    }
 	    if (DgramConnect(ibuf->sock, localhost, server_port) == -1) {
-		error("Can't connect to server %s on port %d\n",
+		error("Can't connect to local server %s on port %d\n",
 		      localhost, server_port);
 		return (false);
 	    }
 	} else {
-	    if ((ibuf->sock = CreateDgramSocket(0)) == -1) {
+	    if ((ibuf->sock = create_dgram_socket(0)) == -1) {
 		error("Could not create socket");
 		exit(1);
 	    }
-	    if (DgramConnect(ibuf->sock, server_addr, server_port) == -1) {
+	    if (DgramConnect(ibuf->sock, server_addr, server_port) == -1 && !onesock) {
 		error("Can't connect to server %s on port %d\n",
 		      server_addr, server_port);
 		return (false);
@@ -386,7 +421,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    case 'K':
 		printf("Enter name of victim: ");
 		fflush(stdout);
-		if (!gets(str)) {
+		if (!getline(str, MAX_LINE-1, stdin)) {
 		    printf("Nothing changed.\n");
 		    continue;
 		}
@@ -397,7 +432,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    case 'R':
 		printf("Enter maximum number of robots: ");
 		fflush(stdout);
-		if (!gets(str)) {
+		if (!getline(str, MAX_LINE-1, stdin)) {
 		    printf("Nothing changed.\n");
 		    continue;
 		}
@@ -411,7 +446,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    case 'M':				/* Send a message to server. */
 		printf("Enter message: ");
 		fflush(stdout);
-		if (!gets(str) || !str[0]) {
+		if (!getline(str, MAX_LINE-1, stdin) || !str[0]) {
 		    printf("No message sent.\n");
 		    continue;
 		}
@@ -426,7 +461,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    case 'D':				/* Shutdown */
 		if (!auto_shutdown) {
 		    printf("Enter delay in seconds or return for cancel: ");
-		    gets(str);
+		    getline(str, MAX_LINE-1, stdin);
 		    /*
 		     * No argument = cancel shutdown = arg_int=0
 		     */
@@ -437,7 +472,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 			    delay = 1;
 
 		    printf("Enter reason: ");
-		    gets(str);
+		    getline(str, MAX_LINE-1, stdin);
 		} else {
 		    strcpy(str, shutdown_reason);
 		    delay = 60;
@@ -449,17 +484,20 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    case 'O':				/* Tune an option. */
 		printf("Enter option: ");
 		fflush(stdout);
-		if (!gets(str) || (len = strlen(str)) == 0) {
+		if (!getline(str, MAX_LINE-1, stdin)
+		    || (len=strlen(str)) == 0) {
 		    printf("Nothing changed.\n");
 		    continue;
 		}
 		printf("Enter new value for %s: ", str);
 		fflush(stdout);
 		strcat(str, ":"); len++;
-		if (!gets(&str[len]) || str[len] == '\0') {
+		if (!getline(&str[len], MAX_LINE-1-len, stdin)
+		    || str[len] == '\0') {
 		    printf("Nothing changed.\n");
 		    continue;
 		}
+		printf("option \"%s\"\n", str); fflush(stdout);
 		Packet_printf(ibuf, "%c%ld%S", OPTION_TUNE_pack, key, str);
 		break;
 
@@ -468,7 +506,11 @@ static bool Process_commands(sockbuf_t *ibuf)
 		 */
 
 	    case 'J':				/* Trying to enter game. */
-		if (str[1] >= '0' && str[1] <= '9') {
+		if (str[1] == '0') {
+		    printf("Team '0' is reserved for robots.");
+		    team = TEAM_NOT_SET;
+		}
+		else if (str[1] > '0' && str[1] <= '9') {
 		    team = str[1] - '0';
 		    printf("Joining team %d\n", team);
 		}
@@ -476,8 +518,17 @@ static bool Process_commands(sockbuf_t *ibuf)
 		    team = TEAM_NOT_SET;
 		    printf("Team set to unspecified\n");
 		}
-		Packet_printf(ibuf, "%c%s%s%s%d", ENTER_GAME_pack,
-			      nick_name, display, hostname, team);
+		else if (str[1] != '\0') {
+		    team = TEAM_NOT_SET;
+		}
+		if (server_version < 0x3430) {
+		    Packet_printf(ibuf, "%c%s%s%s%d", ENTER_GAME_pack,
+				  nick_name, display, hostname, team);
+		} else {
+		    Packet_printf(ibuf, "%c%s%s%s%d", ENTER_QUEUE_pack,
+				  nick_name, display, hostname, team);
+		    time(&qsent);
+		}
 		break;
 
 	    case 'S':				/* Report status. */
@@ -499,7 +550,8 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    case 'T':				/* Set team. */
 		printf("Enter team: ");
 		fflush(stdout);
-		if (!gets(str) || (len = strlen(str)) == 0) {
+		if (!getline(str, MAX_LINE-1, stdin)
+		    || (len = strlen(str)) == 0) {
 		    printf("Nothing changed.\n");
 		}
 		else {
@@ -542,7 +594,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 		    break;
 		}
 	    }
-	    if (write(ibuf->sock, ibuf->buf, ibuf->len) != ibuf->len) {
+	    if (DgramWrite(ibuf->sock, ibuf->buf, ibuf->len) != ibuf->len) {
 		error("Couldn't send request to server.");
 		exit(1);
 	    }
@@ -565,6 +617,8 @@ static bool Process_commands(sockbuf_t *ibuf)
 		return (false);
 	    }
 
+	    SetTimeout(0, 500*1000);
+
 	    /*
 	     * Now try and interpret the result.
 	     */
@@ -624,6 +678,33 @@ static bool Process_commands(sockbuf_t *ibuf)
 		    }
 		    break;
 
+		case ENTER_QUEUE_pack:
+		    if (Packet_scanf(ibuf, "%hu", &qpos) <= 0) {
+			errno = 0;
+			error("Incomplete queue reply from server");
+		    } else {
+			printf("... queued at position %2d\n", qpos);
+		    }
+		    /*
+		     * Acknowledge each 10 seconds that we are still
+		     * interested to be on the waiting queue.
+		     */
+		    if (qsent + 10 <= time(NULL)) {
+			Sockbuf_clear(ibuf);
+			Packet_printf(ibuf, "%u%s%hu", VERSION2MAGIC(server_version),
+				      real_name, GetPortNum(ibuf->sock));
+			Packet_printf(ibuf, "%c%s%s%s%d", ENTER_QUEUE_pack,
+				      nick_name, display, hostname, team);
+			if (DgramWrite(ibuf->sock, ibuf->buf, ibuf->len) != ibuf->len) {
+			    error("Couldn't send request to server.");
+			    exit(1);
+			}
+			time(&qsent);
+		    }
+		    SetTimeout(12, 0);
+		    max_replies = 2;
+		    break;
+
 		case CREDENTIALS_pack:
 		    if (Packet_scanf(ibuf, "%ld", &key) <= 0) {
 			errno = 0;
@@ -680,7 +761,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 		error("Requested operation is undefined, says the server");
 		break;
 	    default:
-		error("Wrong status '%d'", status);
+		error("Server answers with unknown error status '%02x'", status);
 		break;
 	    }
 
@@ -705,7 +786,6 @@ static bool Process_commands(sockbuf_t *ibuf)
 		}
 	    }
 
-	    SetTimeout(0, 500*1000);
 	} while (--max_replies > 0 && SocketReadable(ibuf->sock));
 
 	/*
@@ -735,7 +815,7 @@ static bool Connect_to_server(void)
     }
     result = Process_commands(&ibuf);
     if (ibuf.sock != -1) {
-	SocketClose(ibuf.sock);
+	close_dgram_socket(ibuf.sock);
     }
     Sockbuf_cleanup(&ibuf);
 
@@ -754,7 +834,7 @@ static bool Contact_servers(int count, char **servers)
     sockbuf_t		sbuf;			/* contact buffer */
 
 
-    if ((fd = CreateDgramSocket(0)) == -1) {
+    if ((fd = create_dgram_socket(0)) == -1) {
 	error("Could not create connection socket");
 	exit(1);
     }
@@ -811,7 +891,7 @@ static bool Contact_servers(int count, char **servers)
 	}
     }
     Sockbuf_cleanup(&sbuf);
-    close(fd);
+    close_dgram_socket(fd);
 
     return (connected ? true : false);
 }
@@ -823,12 +903,12 @@ static bool Contact_servers(int count, char **servers)
 int main(int argc, char *argv[])
 {
     struct passwd	*pwent;
-
+    int			result;
 
     /*
      * --- Output copyright notice ---
      */
-    printf("  Copyright " COPYRIGHT ".\n"
+    printf("  " COPYRIGHT ".\n"
 	   "  " TITLE " comes with ABSOLUTELY NO WARRANTY; "
 	      "for details see the\n"
 	   "  provided LICENSE file.\n\n");
@@ -898,7 +978,18 @@ int main(int argc, char *argv[])
 	exit (-1);
 #endif
 
-    if (Contact_servers(argc - 1, &argv[1])) {
+#if 1
+    result = Contact_servers(argc - 1, &argv[1]);
+#else
+    if (auto_connect) {
+	result = Contact_servers(argc - 1, &argv[1]);
+    }
+    else {
+	extern int metaclient(int, char **);
+	result = metaclient(argc - 1, &argv[1]);
+    }
+#endif
+    if (result == 1) {
 	Join(server_addr, server_name, login_port,
 	     real_name, nick_name, team,
 	     display, server_version);
